__author__ = 'Paul Gundarapu'
__author_email__ = 'leogps@outlook.com'
__version__ = '0.1.14'
__license__ = 'MIT'
__description__ = 'Simple, zero-configuration command-line static HTTP server.'
__long_description__ = """
Sipper is a simple, zero-configuration command-line static HTTP server. It is built on bottle:
https://github.com/bottlepy/bottle | https://bottlepy.org/docs/dev/ It aims to provide the same value provided by
http-server tool written in nodejs: https://github.com/http-party/http-server |
https://www.npmjs.com/package/http-server

Homepage and documentation: https://github.com/leogps/bottle-sipper

Copyright (c) 2023, Paul Gundarapu. License: MIT (see LICENSE for details)
"""
