<tr id="{{file_details.hash}}" class="data-row">
  <td class="icon">
    <i class="icon {{ file_details.file_icon_style_class }}"></i>
  </td>
  <td class="file-permissions">
    {{ file_details.file_permissions }}
  </td>
  <td class="modified-date">
    {{ file_details.last_modified_date }}
  </td>
  <td class="file-size">
    {{ file_details.file_size }}
  </td>
  <td class="file-link display-name">
    <a href="{{ file_details.file_link }}">{{ file_details.file_name }}</a>
  </td>
</tr>
